@@ -1,4 +1,5 @@
 # Copyright 2019 Shift Cryptosecurity AG
+# Copyright 2020 Shift Crypto AG
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +19,7 @@ import os
 import sys
 import time
 from datetime import datetime
-from typing import Optional, List, Dict, Tuple, Any, Generator, Union
+from typing import Optional, List, Dict, Tuple, Any, Generator, Union, Sequence
 from typing_extensions import TypedDict
 
 from bitbox02.communication import (
@@ -73,8 +74,8 @@ class BTCPrevTxOutputType(TypedDict):
 class BTCPrevTxType(TypedDict):
     version: int
     locktime: int
-    inputs: List[BTCPrevTxInputType]
-    outputs: List[BTCPrevTxOutputType]
+    inputs: Sequence[BTCPrevTxInputType]
+    outputs: Sequence[BTCPrevTxOutputType]
 
 
 class BTCInputType(TypedDict):
@@ -82,19 +83,21 @@ class BTCInputType(TypedDict):
     prev_out_index: int
     prev_out_value: int
     sequence: int
-    keypath: List[int]
+    keypath: Sequence[int]
+    script_config_index: int
     prev_tx: BTCPrevTxType
 
 
 class BTCOutputInternal:
     # TODO: Use NamedTuple, but not playing well with protobuf types.
 
-    def __init__(self, keypath: List[int], value: int):
+    def __init__(self, keypath: Sequence[int], value: int, script_config_index: int):
         """
         keypath: keypath to the change output.
         """
         self.keypath = keypath
         self.value = value
+        self.script_config_index = script_config_index
 
 
 class BTCOutputExternal:
@@ -261,7 +264,7 @@ class BitBox02(BitBoxCommonAPI):
 
     def btc_xpub(
         self,
-        keypath: List[int],
+        keypath: Sequence[int],
         coin: btc.BTCCoin = btc.BTC,
         xpub_type: btc.BTCPubRequest.XPubType = btc.BTCPubRequest.XPUB,
         display: bool = True,
@@ -279,7 +282,7 @@ class BitBox02(BitBoxCommonAPI):
 
     def btc_address(
         self,
-        keypath: List[int],
+        keypath: Sequence[int],
         coin: btc.BTCCoin = btc.BTC,
         script_config: btc.BTCScriptConfig = btc.BTCScriptConfig(
             simple_type=btc.BTCScriptConfig.P2WPKH
@@ -300,7 +303,7 @@ class BitBox02(BitBoxCommonAPI):
         return self._msg_query(request).pub.pub
 
     def btc_is_script_config_registered(
-        self, coin: btc.BTCCoin, script_config: btc.BTCScriptConfig, keypath: List[int]
+        self, coin: btc.BTCCoin, script_config: btc.BTCScriptConfig, keypath: Sequence[int]
     ) -> bool:
         """
         Returns True if the script config / account is already registered.
@@ -319,7 +322,11 @@ class BitBox02(BitBoxCommonAPI):
         ).is_script_config_registered.is_registered
 
     def btc_register_script_config(
-        self, coin: btc.BTCCoin, script_config: btc.BTCScriptConfig, keypath: List[int], name: str
+        self,
+        coin: btc.BTCCoin,
+        script_config: btc.BTCScriptConfig,
+        keypath: Sequence[int],
+        name: str,
     ) -> None:
         """
         Raises Bitbox02Exception with ERR_USER_ABORT on user abort.
@@ -350,13 +357,12 @@ class BitBox02(BitBoxCommonAPI):
     def btc_sign(
         self,
         coin: btc.BTCCoin,
-        script_config: btc.BTCScriptConfig,
-        keypath_account: List[int],
-        inputs: List[BTCInputType],
-        outputs: List[BTCOutputType],
+        script_configs: Sequence[btc.BTCScriptConfigWithKeypath],
+        inputs: Sequence[BTCInputType],
+        outputs: Sequence[BTCOutputType],
         version: int = 1,
         locktime: int = 0,
-    ) -> List[Tuple[int, bytes]]:
+    ) -> Sequence[Tuple[int, bytes]]:
         """
         coin: the first element of all provided keypaths must match the coin:
         - BTC: 0 + HARDENED
@@ -388,8 +394,7 @@ class BitBox02(BitBoxCommonAPI):
         request.btc_sign_init.CopyFrom(
             btc.BTCSignInitRequest(
                 coin=coin,
-                script_config=script_config,
-                keypath_account=keypath_account,
+                script_configs=script_configs,
                 version=version,
                 num_inputs=len(inputs),
                 num_outputs=len(outputs),
@@ -410,6 +415,7 @@ class BitBox02(BitBoxCommonAPI):
                         prevOutValue=tx_input["prev_out_value"],
                         sequence=tx_input["sequence"],
                         keypath=tx_input["keypath"],
+                        script_config_index=tx_input["script_config_index"],
                     )
                 )
                 next_response = self._msg_query(
@@ -468,7 +474,10 @@ class BitBox02(BitBoxCommonAPI):
                 if isinstance(tx_output, BTCOutputInternal):
                     request.btc_sign_output.CopyFrom(
                         btc.BTCSignOutputRequest(
-                            ours=True, value=tx_output.value, keypath=tx_output.keypath
+                            ours=True,
+                            value=tx_output.value,
+                            keypath=tx_output.keypath,
+                            script_config_index=tx_output.script_config_index,
                         )
                     )
                 elif isinstance(tx_output, BTCOutputExternal):
@@ -526,7 +535,7 @@ class BitBox02(BitBoxCommonAPI):
         response = self._msg_query(request, expected_response="fingerprint")
         return response.fingerprint.fingerprint
 
-    def electrum_encryption_key(self, keypath: List[int]) -> str:
+    def electrum_encryption_key(self, keypath: Sequence[int]) -> str:
         """
         This call fetches the xpub used for the electrum wallet encryption
         """
@@ -578,7 +587,7 @@ class BitBox02(BitBoxCommonAPI):
 
     def eth_pub(
         self,
-        keypath: List[int],
+        keypath: Sequence[int],
         coin: eth.ETHCoin = eth.ETH,
         output_type: eth.ETHPubRequest.OutputType = eth.ETHPubRequest.ADDRESS,
         display: bool = True,
@@ -602,7 +611,7 @@ class BitBox02(BitBoxCommonAPI):
         return self._eth_msg_query(request, expected_response="pub").pub.pub
 
     def eth_sign(
-        self, transaction: bytes, keypath: List[int], coin: eth.ETHCoin = eth.ETH
+        self, transaction: bytes, keypath: Sequence[int], coin: eth.ETHCoin = eth.ETH
     ) -> bytes:
         """
         transaction should be given as a full rlp encoded eth transaction.
@@ -624,7 +633,9 @@ class BitBox02(BitBoxCommonAPI):
         )
         return self._eth_msg_query(request, expected_response="sign").sign.signature
 
-    def eth_sign_msg(self, msg: bytes, keypath: List[int], coin: eth.ETHCoin = eth.ETH) -> bytes:
+    def eth_sign_msg(
+        self, msg: bytes, keypath: Sequence[int], coin: eth.ETHCoin = eth.ETH
+    ) -> bytes:
         """
         Signs message, the msg will be prefixed with "\x19Ethereum message\n" + len(msg) in the
         hardware
