@@ -21,7 +21,7 @@ import base64
 import binascii
 import hashlib
 import time
-from typing import Callable, Optional, List, Dict, Tuple, Union
+from typing import Callable, Optional, Dict, Tuple, Union, Sequence
 from typing_extensions import TypedDict
 
 import ecdsa
@@ -83,7 +83,7 @@ class AttestationPubkeyInfo(TypedDict):
     accepted_bootloader_hash: Optional[bytes]
 
 
-ATTESTATION_PUBKEYS: List[AttestationPubkeyInfo] = [
+ATTESTATION_PUBKEYS: Sequence[AttestationPubkeyInfo] = [
     {
         "pubkey": binascii.unhexlify(
             "04074ff1273b36c24e80fe3d59e0e897a81732d3f8e9cd07e17e9fc06319cd16b"
@@ -144,10 +144,10 @@ OP_NOISE_MSG = b"n"
 RESPONSE_SUCCESS = b"\x00"
 RESPONSE_FAILURE = b"\x01"
 
-MIN_SUPPORTED_BITBOX02_MULTI_FIRMWARE_VERSION = semver.VersionInfo(8, 0, 0)
-MIN_SUPPORTED_BITBOX02_BTCONLY_FIRMWARE_VERSION = semver.VersionInfo(8, 0, 0)
-MIN_UNSUPPORTED_BITBOX02_MULTI_FIRMWARE_VERSION = semver.VersionInfo(9, 0, 0)
-MIN_UNSUPPORTED_BITBOX02_BTCONLY_FIRMWARE_VERSION = semver.VersionInfo(9, 0, 0)
+MIN_SUPPORTED_BITBOX02_MULTI_FIRMWARE_VERSION = semver.VersionInfo(9, 0, 0)
+MIN_SUPPORTED_BITBOX02_BTCONLY_FIRMWARE_VERSION = semver.VersionInfo(9, 0, 0)
+MIN_UNSUPPORTED_BITBOX02_MULTI_FIRMWARE_VERSION = semver.VersionInfo(10, 0, 0)
+MIN_UNSUPPORTED_BITBOX02_BTCONLY_FIRMWARE_VERSION = semver.VersionInfo(10, 0, 0)
 
 
 class Platform(enum.Enum):
